#!/usr/bin/env python

"""
synthetic2.py
Convert chi2_* files to synthetic data and add Gaussian noise.

"""

__author__ = "Miroslav Broz (miroslav.broz@email.cz)"
__version__ = "Apr 4th 2017"

import re
import math
import random
import sys

deg = math.pi/180.
rad = 180./math.pi
arcsec = deg/3600.
au = 1.49597870700e11  # m, from IAU 2012
pc = 648000.0/math.pi*au  # m, from IAU 2015

########################################################################

def read_chi2(filename):
    """Read chi2_*.dat files (every 1st line)"""
    fin = open(filename, "r")
    txt = []
    j = 0
    for line in fin.readlines():
        if not re.match("^ *#", line) and len(line) > 1:
            if j == 0:
                txt.append(line)
                j = 1
            else:
                j = 0
    fin.close()
    return txt

########################################################################

random.seed(-1)

def noise(line, i, j):
    """Add Gaussian noise from column j to data in column i"""

    l = line.split()
    S = float(l[i])
    sigma = float(l[j])
    S = random.normalvariate(S, sigma)

    n = len(l)
    fmt = "%s "*i + "%22.16e %22.16e" + " %s"*(n-j-1) + "\n"
    return fmt % tuple(l[:i] + [S, sigma] + l[j+1:])

########################################################################

txt = read_chi2("../chi2_LC.dat")

i = 0
j = 0
for line in txt:
    l = line.split()
    iband = l[3] 
    if iband != i:
        if j > 0:
            fout.close()
        j += 1
        fout = open("chi2_LC_" + str(j) + ".syn", "w")
        i = iband

    line = noise(line, 1, 2)
    fout.write(line)

fout.close()

########################################################################

txt = read_chi2("../chi2_RV.dat")

i = 0
j = 0
for line in txt:
    l = line.split()
    ibod = l[3] 
    if ibod != i:
        if j > 0:
            fout.close()
        j += 1
        fout = open("chi2_RV_" + str(j) + ".syn", "w")
        i = ibod

    line = noise(line, 1, 2)
    fout.write(line)

fout.close()

########################################################################

txt = read_chi2("../chi2_SKY.dat")

i = 0
j = 0
for line in txt:
    l = line.split()
    ibod = l[6] 
    if ibod != i:
        if j > 0:
            fout.close()
        j += 1
        fout = open("chi2_SKY_" + str(ibod) + ".syn", "w")
        i = ibod

    rho = float(l[1])
    theta = float(l[2])*deg
    major = float(l[3])
    minor = float(l[4])
    PA_ellipse = float(l[5])*deg

    dx = random.normalvariate(0.0, major)
    dy = random.normalvariate(0.0, minor)
    phi = -(PA_ellipse + math.pi/2.0)
    dx_ =  dx*math.cos(phi) + dy*math.sin(phi)
    dy_ = -dx*math.sin(phi) + dy*math.cos(phi)

    x = -rho*math.sin(theta)
    y = rho*math.cos(theta)
    x += dx_
    y += dy_
    rho = math.sqrt(x**2 + y**2)
    theta = math.atan2(-x, y)
    if theta < 0.0:
        theta += 2.0*math.pi

    n = len(l)
    i_ = 1
    j_ = 2
    fmt = "%s "*i_ + "%22.16e %22.16e" + " %s"*(n-j_-1) + "\n"
    line = fmt % tuple(l[:i_] + [rho, theta*rad] + l[j_+1:])

    fout.write(line)

fout.close()

########################################################################

files = "chi2_VIS.dat chi2_SYN.dat chi2_SED.dat ".split()
i = [ 5, 2, 2 ]
j = [ 6, 3, 3 ]
k = 0

for filename in files:

    txt = read_chi2("../" + filename)
    fout = open(filename.replace(".dat", ".syn"), "w")

    for line in txt:
       line = noise(line, i[k], j[k])
       fout.write(line)

    fout.close()
    k += 1

########################################################################

txt = read_chi2("../chi2_CLO.dat")
fout = open("chi2_CLO.syn", "w")

for line in txt:
    line = noise(line, 7, 8)
    line = noise(line, 9, 10)
    fout.write(line)

fout.close()

########################################################################

